@@ -1,18 +1,21 @@
-#!/usr/bin/env  python2
+#!/usr/bin/env python
 # License: GPLv3 Copyright: 2008, Kovid Goyal <kovid at kovidgoyal.net>
-from __future__ import absolute_import, division, print_function, unicode_literals
+
+
+import textwrap
 
 from PyQt5.Qt import (
     QBrush, QCheckBox, QCoreApplication, QDialog, QGridLayout, QHBoxLayout, QIcon,
     QKeySequence, QLabel, QListView, QModelIndex, QPalette, QPixmap, QPushButton,
     QShortcut, QSize, QSplitter, Qt, QTimer, QToolButton, QVBoxLayout, QWidget,
-    pyqtSignal
+    pyqtSignal, QApplication
 )
 
 from calibre import fit_image
 from calibre.gui2 import NO_URL_FORMATTING, gprefs
 from calibre.gui2.book_details import (
-    css, details_context_menu_event, render_html, set_html
+    create_open_cover_with_menu, css, details_context_menu_event, render_html,
+    set_html
 )
 from calibre.gui2.ui import get_gui
 from calibre.gui2.widgets import CoverView
@@ -20,6 +23,27 @@ from calibre.gui2.widgets2 import Dialog, HTMLDisplay
 from polyglot.builtins import unicode_type
 
 
+class Cover(CoverView):
+
+    open_with_requested = pyqtSignal(object)
+    choose_open_with_requested = pyqtSignal()
+
+    def build_context_menu(self):
+        ans = CoverView.build_context_menu(self)
+        create_open_cover_with_menu(self, ans)
+        return ans
+
+    def open_with(self, entry):
+        self.open_with_requested.emit(entry)
+
+    def choose_open_with(self):
+        self.choose_open_with_requested.emit()
+
+    def mouseDoubleClickEvent(self, ev):
+        ev.accept()
+        self.open_with_requested.emit(None)
+
+
 class Configure(Dialog):
 
     def __init__(self, db, parent=None):
@@ -95,6 +119,7 @@ class Details(HTMLDisplay):
 class BookInfo(QDialog):
 
     closed = pyqtSignal(object)
+    open_cover_with = pyqtSignal(object, object)
 
     def __init__(self, parent, view, row, link_delegate):
         QDialog.__init__(self, parent)
@@ -107,9 +132,11 @@ class BookInfo(QDialog):
         self.setLayout(l)
         l.addWidget(self.splitter)
 
-        self.cover = CoverView(self, show_size=gprefs['bd_overlay_cover_size'])
+        self.cover = Cover(self, show_size=gprefs['bd_overlay_cover_size'])
         self.cover.resizeEvent = self.cover_view_resized
         self.cover.cover_changed.connect(self.cover_changed)
+        self.cover.open_with_requested.connect(self.open_with)
+        self.cover.choose_open_with_requested.connect(self.choose_open_with)
         self.cover_pixmap = None
         self.cover.sizeHint = self.details_size_hint
         self.splitter.addWidget(self.cover)
@@ -148,6 +175,7 @@ class BookInfo(QDialog):
         l2.addWidget(self.next_button, l2.rowCount() - 1, 1)
 
         self.view = view
+        self.path_to_book = None
         self.current_row = None
         self.refresh(row)
         self.view.model().new_bookdisplay_data.connect(self.slave)
@@ -168,7 +196,7 @@ class BookInfo(QDialog):
         saved_layout = gprefs.get('book_info_dialog_layout', None)
         if saved_layout is not None:
             try:
-                self.restoreGeometry(saved_layout[0])
+                QApplication.instance().safe_restore_geometry(self, saved_layout[0])
                 self.splitter.restoreState(saved_layout[1])
             except Exception:
                 pass
@@ -254,9 +282,14 @@ class BookInfo(QDialog):
     def update_cover_tooltip(self):
         tt = ''
         if self.marked:
-            tt = _('This book is marked') if self.marked in {True, 'true'} else _(
+            tt += _('This book is marked') if self.marked in {True, 'true'} else _(
                 'This book is marked as: %s') % self.marked
             tt += '\n\n'
+
+        if self.path_to_book is not None:
+            tt += textwrap.fill(_('Path: {}').format(self.path_to_book))
+            tt += '\n\n'
+
         if self.cover_pixmap is not None:
             sz = self.cover_pixmap.size()
             tt += _('Cover size: %(width)d x %(height)d pixels')%dict(width=sz.width(), height=sz.height())
@@ -279,6 +312,7 @@ class BookInfo(QDialog):
         self.current_row = row
         self.setWindowTitle(mi.title)
         self.cover_pixmap = QPixmap.fromImage(mi.cover_data[1])
+        self.path_to_book = getattr(mi, 'path', None)
         try:
             dpr = self.devicePixelRatioF()
         except AttributeError:
@@ -291,6 +325,16 @@ class BookInfo(QDialog):
         self.cover.setBackgroundBrush(self.marked_brush if mi.marked else self.normal_brush)
         self.update_cover_tooltip()
 
+    def open_with(self, entry):
+        id_ = self.view.model().id(self.current_row)
+        self.open_cover_with.emit(id_, entry)
+
+    def choose_open_with(self):
+        from calibre.gui2.open_with import choose_program
+        entry = choose_program('cover_image', self)
+        if entry is not None:
+            self.open_with(entry)
+
 
 if __name__ == '__main__':
     from calibre.gui2 import Application
