@@ -1,10 +1,10 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
+
 import os
 import re
-import socket
 import sys
 import time
 import traceback
@@ -12,24 +12,26 @@ import traceback
 import apsw
 from PyQt5.Qt import QCoreApplication, QIcon, QObject, QTimer
 
-from calibre import force_unicode, plugins, prints
+from calibre import force_unicode, prints
 from calibre.constants import (
     DEBUG, MAIN_APP_UID, __appname__, filesystem_encoding, get_portable_base,
-    islinux, isosx, iswindows
+    islinux, ismacos, iswindows
 )
 from calibre.gui2 import (
     Application, choose_dir, error_dialog, gprefs, initialize_file_icon_provider,
-    question_dialog, set_app_uid, setup_gui_option_parser
+    question_dialog, setup_gui_option_parser
 )
+from calibre.gui2.listener import send_message_in_process
 from calibre.gui2.main_window import option_parser as _option_parser
 from calibre.gui2.splash_screen import SplashScreen
 from calibre.utils.config import dynamic, prefs
-from calibre.utils.ipc import RC, gui_socket_address
-from calibre.utils.lock import singleinstance
+from calibre.utils.lock import SingleInstance
 from calibre.utils.monotonic import monotonic
+from polyglot.builtins import as_bytes, environ_item, range, unicode_type
 
+after_quit_actions = {'debug_on_restart': False, 'restart_after_quit': False}
 if iswindows:
-    winutil = plugins['winutil'][0]
+    from calibre_extensions import winutil
 
 
 class AbortInit(Exception):
@@ -38,10 +40,18 @@ class AbortInit(Exception):
 
 def option_parser():
     parser = _option_parser(_('''\
-%prog [options] [path_to_ebook]
+%prog [options] [path_to_ebook or calibre url ...]
 
 Launch the main calibre Graphical User Interface and optionally add the e-book at
-path_to_ebook to the database.
+path_to_ebook to the database. You can also specify calibre URLs to perform various
+different actions, than just adding books. For example:
+
+calibre://view-book/test_library/1842/epub
+
+Will open the book with id 1842 in the EPUB format from the library
+"test_library" in the calibre viewer. Library names are the folder names of the
+libraries with spaces replaced by underscores. A full description of the
+various URL based actions is in the User Manual.
 '''))
     parser.add_option('--with-library', default=None, action='store',
                       help=_('Use the library located at the specified path.'))
@@ -69,9 +79,9 @@ def find_portable_library():
         return
     import glob
     candidates = [os.path.basename(os.path.dirname(x)) for x in glob.glob(
-        os.path.join(base, u'*%smetadata.db'%os.sep))]
+        os.path.join(base, '*%smetadata.db'%os.sep))]
     if not candidates:
-        candidates = [u'Calibre Library']
+        candidates = ['Calibre Library']
     lp = prefs['library_path']
     if not lp:
         lib = os.path.join(base, candidates[0])
@@ -109,7 +119,8 @@ def init_qt(args):
             prefs.set('library_path', os.path.abspath(libpath))
             prints('Using library at', prefs['library_path'])
     override = 'calibre-gui' if islinux else None
-    app = Application(args, override_program_name=override)
+    app = Application(args, override_program_name=override, windows_app_uid=MAIN_APP_UID)
+
     app.file_event_hook = EventAccumulator()
     try:
         is_x11 = app.platformName() == 'xcb'
@@ -128,16 +139,16 @@ def get_default_library_path():
     fname = _('Calibre Library')
     if iswindows:
         fname = 'Calibre Library'
-    if isinstance(fname, unicode):
+    if isinstance(fname, unicode_type):
         try:
-            fname = fname.encode(filesystem_encoding)
-        except:
+            fname.encode(filesystem_encoding)
+        except Exception:
             fname = 'Calibre Library'
-    x = os.path.expanduser('~'+os.sep+fname)
+    x = os.path.expanduser(os.path.join('~', fname))
     if not os.path.exists(x):
         try:
             os.makedirs(x)
-        except:
+        except Exception:
             x = os.path.expanduser('~')
     return x
 
@@ -148,7 +159,7 @@ def get_library_path(gui_runner):
         base = os.path.expanduser('~')
         if not base or not os.path.exists(base):
             from PyQt5.Qt import QDir
-            base = unicode(QDir.homePath()).replace('/', os.sep)
+            base = unicode_type(QDir.homePath()).replace('/', os.sep)
         candidate = gui_runner.choose_dir(base)
         if not candidate:
             candidate = os.path.join(base, 'Calibre Library')
@@ -171,18 +182,19 @@ def repair_library(library_path):
 
 
 def windows_repair(library_path=None):
-    from binascii import hexlify, unhexlify
-    import cPickle, subprocess
+    import subprocess
+    from calibre.utils.serialize import json_dumps, json_loads
+    from polyglot.binary import as_hex_unicode, from_hex_bytes
     if library_path:
-        library_path = hexlify(cPickle.dumps(library_path, -1))
+        library_path = as_hex_unicode(json_dumps(library_path))
         winutil.prepare_for_restart()
-        os.environ['CALIBRE_REPAIR_CORRUPTED_DB'] = library_path
+        os.environ['CALIBRE_REPAIR_CORRUPTED_DB'] = environ_item(library_path)
         subprocess.Popen([sys.executable])
     else:
         try:
             app = Application([])
             from calibre.gui2.dialogs.restore_library import repair_library_at
-            library_path = cPickle.loads(unhexlify(os.environ.pop('CALIBRE_REPAIR_CORRUPTED_DB')))
+            library_path = json_loads(from_hex_bytes(os.environ.pop('CALIBRE_REPAIR_CORRUPTED_DB')))
             done = repair_library_at(library_path, wait_time=4)
         except Exception:
             done = False
@@ -206,10 +218,10 @@ class GuiRunner(QObject):
     '''Make sure an event loop is running before starting the main work of
     initialization'''
 
-    def __init__(self, opts, args, actions, listener, app, gui_debug=None):
+    def __init__(self, opts, args, actions, app, gui_debug=None):
         self.startup_time = monotonic()
         self.timed_print('Starting up...')
-        self.opts, self.args, self.listener, self.app = opts, args, listener, app
+        self.opts, self.args, self.app = opts, args, app
         self.gui_debug = gui_debug
         self.actions = actions
         self.main = None
@@ -229,7 +241,7 @@ class GuiRunner(QObject):
             self.splash_screen.show_message(_('Initializing user interface...'))
         try:
             with gprefs:  # Only write gui.json after initialization is complete
-                main.initialize(self.library_path, db, self.listener, self.actions)
+                main.initialize(self.library_path, db, self.actions)
         finally:
             self.timed_print('main UI initialized...')
             if self.splash_screen is not None:
@@ -257,7 +269,7 @@ class GuiRunner(QObject):
             error_dialog(self.splash_screen, title, msg, det_msg=det_msg, show=True)
 
     def initialization_failed(self):
-        print 'Catastrophic failure initializing GUI, bailing out...'
+        print('Catastrophic failure initializing GUI, bailing out...')
         QCoreApplication.exit(1)
         raise SystemExit(1)
 
@@ -287,10 +299,13 @@ class GuiRunner(QObject):
         try:
             self.start_gui(db)
         except Exception:
+            try:
+                details = traceback.format_exc()
+            except Exception:
+                details = ''
             self.show_error(_('Startup error'), _(
                 'There was an error during {0} startup. Parts of {0} may not function.'
-                ' Click Show details to learn more.').format(__appname__),
-                         det_msg=traceback.format_exc())
+                ' Click Show details to learn more.').format(__appname__), det_msg=details)
 
     def initialize_db(self):
         from calibre.db.legacy import LibraryDatabase
@@ -354,27 +369,26 @@ def run_in_debug_mode():
     import tempfile, subprocess
     fd, logpath = tempfile.mkstemp('.txt')
     os.close(fd)
-    os.environ[b'CALIBRE_RESTARTING_FROM_GUI'] = b'1'
     run_calibre_debug(
-        '--gui-debug', logpath, stdout=lopen(logpath, 'w'),
-        stderr=subprocess.STDOUT, stdin=lopen(os.devnull, 'r'))
+        '--gui-debug', logpath, stdout=lopen(logpath, 'wb'),
+        stderr=subprocess.STDOUT, stdin=lopen(os.devnull, 'rb'))
 
 
-def shellquote(s):
-    return "'" + s.replace("'", "'\\''") + "'"
+def run_gui(opts, args, app, gui_debug=None):
+    with SingleInstance('db') as si:
+        if not si:
+            ext = '.exe' if iswindows else ''
+            error_dialog(None, _('Cannot start calibre'), _(
+                'Another calibre program that can modify calibre libraries, such as,'
+                ' {0} or {1} is already running. You must first shut it down, before'
+                ' starting the main calibre program. If you are sure no such'
+                ' program is running, try restarting your computer.').format(
+                    'calibre-server' + ext, 'calibredb' + ext), show=True)
+            return 1
+        run_gui_(opts, args, app, gui_debug)
 
 
-def run_gui(opts, args, listener, app, gui_debug=None):
-    si = singleinstance('db')
-    if not si:
-        ext = '.exe' if iswindows else ''
-        error_dialog(None, _('Cannot start calibre'), _(
-            'Another calibre program that can modify calibre libraries, such as,'
-            ' {} or {} is already running. You must first shut it down, before'
-            ' starting the main calibre program. If you are sure no such'
-            ' program is running, try restarting your computer.').format(
-                'calibre-server' + ext, 'calibredb' + ext), show=True)
-        return 1
+def run_gui_(opts, args, app, gui_debug=None):
     initialize_file_icon_provider()
     app.load_builtin_fonts(scan_for_fonts=True)
     if not dynamic.get('welcome_wizard_was_run', False):
@@ -382,32 +396,18 @@ def run_gui(opts, args, listener, app, gui_debug=None):
         wizard().exec_()
         dynamic.set('welcome_wizard_was_run', True)
     from calibre.gui2.ui import Main
-    if isosx:
+    if ismacos:
         actions = tuple(Main.create_application_menubar())
     else:
         actions = tuple(Main.get_menubar_actions())
-    runner = GuiRunner(opts, args, actions, listener, app, gui_debug=gui_debug)
+    runner = GuiRunner(opts, args, actions, app, gui_debug=gui_debug)
     ret = app.exec_()
     if getattr(runner.main, 'run_wizard_b4_shutdown', False):
         from calibre.gui2.wizard import wizard
         wizard().exec_()
     if getattr(runner.main, 'restart_after_quit', False):
-        e = sys.executable if getattr(sys, 'frozen', False) else sys.argv[0]
-        if getattr(runner.main, 'debug_on_restart', False) or gui_debug is not None:
-            run_in_debug_mode()
-        else:
-            import subprocess
-            if hasattr(sys, 'frameworks_dir'):
-                app = os.path.dirname(os.path.dirname(os.path.realpath(sys.frameworks_dir)))
-                prints('Restarting with:', app)
-                subprocess.Popen('sleep 3s; open ' + shellquote(app), shell=True)
-            else:
-                os.environ[b'CALIBRE_RESTARTING_FROM_GUI'] = b'1'
-                if iswindows and hasattr(winutil, 'prepare_for_restart'):
-                    winutil.prepare_for_restart()
-                args = ['-g'] if os.path.splitext(e)[0].endswith('-debug') else []
-                prints('Restarting with:', ' '.join([e] + args))
-                subprocess.Popen([e] + args)
+        after_quit_actions['restart_after_quit'] = True
+        after_quit_actions['debug_on_restart'] = getattr(runner.main, 'debug_on_restart', False) or gui_debug is not None
     else:
         if iswindows:
             try:
@@ -415,7 +415,6 @@ def run_gui(opts, args, listener, app, gui_debug=None):
             except:
                 pass
     if getattr(runner.main, 'gui_debug', None) is not None:
-        e = sys.executable if getattr(sys, 'frozen', False) else sys.argv[0]
         debugfile = runner.main.gui_debug
         from calibre.gui2 import open_local_file
         if iswindows:
@@ -432,82 +431,72 @@ def run_gui(opts, args, listener, app, gui_debug=None):
 singleinstance_name = 'GUI'
 
 
-def cant_start(msg=_('If you are sure it is not running')+', ',
-               det_msg=_('Timed out waiting for response from running calibre'),
-               listener_failed=False):
-    base = '<p>%s</p><p>%s %s'
-    where = __appname__ + ' '+_('may be running in the system tray, in the')+' '
-    if isosx:
-        where += _('upper right region of the screen.')
-    else:
-        where += _('lower right region of the screen.')
-    if iswindows or islinux:
-        what = _('try rebooting your computer.')
-    else:
-        if listener_failed:
-            path = gui_socket_address()
-        else:
-            from calibre.utils.lock import singleinstance_path
-            path = singleinstance_path(singleinstance_name)
-        what = _('try deleting the file: "%s"') % path
-
-    info = base%(where, msg, what)
-    error_dialog(None, _('Cannot Start ')+__appname__,
-        '<p>'+(_('%s is already running.')%__appname__)+'</p>'+info, det_msg=det_msg, show=True)
-
-    raise SystemExit(1)
+def send_message(msg):
+    try:
+        send_message_in_process(msg)
+    except Exception as err:
+        print(_('Failed to contact running instance of calibre'), file=sys.stderr, flush=True)
+        print(err, file=sys.stderr, flush=True)
+        error_dialog(None, _('Contacting calibre failed'), _(
+            'Failed to contact running instance of calibre, try restarting calibre'),
+            det_msg=str(err) + '\n\n' + repr(msg), show=True)
+        return False
+    return True
 
 
-def build_pipe(print_error=True):
-    t = RC(print_error=print_error)
-    t.start()
-    t.join(3.0)
-    if t.is_alive():
-        cant_start()
-        raise SystemExit(1)
-    return t
-
-
-def shutdown_other(rc=None):
-    if rc is None:
-        rc = build_pipe(print_error=False)
-        if rc.conn is None:
-            prints(_('No running calibre found'))
-            return  # No running instance found
-    rc.conn.send('shutdown:')
-    prints(_('Shutdown command sent, waiting for shutdown...'))
-    for i in xrange(50):
-        if singleinstance(singleinstance_name):
-            return
-        time.sleep(0.1)
-    prints(_('Failed to shutdown running calibre instance'))
-    raise SystemExit(1)
+def shutdown_other():
+    if send_message('shutdown:'):
+        print(_('Shutdown command sent, waiting for shutdown...'), flush=True)
+        for i in range(50):
+            with SingleInstance(singleinstance_name) as si:
+                if si:
+                    return
+            time.sleep(0.1)
+        raise SystemExit(_('Failed to shutdown running calibre instance'))
 
 
 def communicate(opts, args):
-    t = build_pipe()
     if opts.shutdown_running_calibre:
-        shutdown_other(t)
+        shutdown_other()
     else:
         if len(args) > 1:
             args[1:] = [os.path.abspath(x) if os.path.exists(x) else x for x in args[1:]]
         import json
-        t.conn.send('launched:'+json.dumps(args))
-    t.conn.close()
+        if not send_message(b'launched:'+as_bytes(json.dumps(args))):
+            raise SystemExit(_('Failed to contact running instance of calibre'))
     raise SystemExit(0)
 
 
-def create_listener():
-    if islinux:
-        from calibre.utils.ipc.server import LinuxListener as Listener
+def restart_after_quit():
+    if iswindows:
+        # detach the stdout/stderr/stdin handles
+        winutil.prepare_for_restart()
+    if after_quit_actions['debug_on_restart']:
+        run_in_debug_mode()
+        return
+    if hasattr(sys, 'frameworks_dir'):
+        app = os.path.dirname(os.path.dirname(os.path.realpath(sys.frameworks_dir)))
+        from calibre.debug import run_calibre_debug
+        prints('Restarting with:', app)
+        run_calibre_debug('-c', 'import sys, os, time; time.sleep(3); os.execlp("open", "open", sys.argv[-1])', app)
     else:
-        from multiprocessing.connection import Listener
-    return Listener(address=gui_socket_address())
+        import subprocess
+        if hasattr(sys, 'run_local'):
+            cmd = [sys.run_local]
+            if DEBUG:
+                cmd += ['calibre-debug', '-g']
+            else:
+                cmd.append('calibre')
+        else:
+            e = sys.executable if getattr(sys, 'frozen', False) else sys.argv[0]
+            cmd = [e]
+            if os.path.splitext(e)[0].endswith('-debug'):
+                cmd.append('-g')
+        prints('Restarting with:', ' '.join(cmd))
+        subprocess.Popen(cmd)
 
 
 def main(args=sys.argv):
-    if os.environ.pop(b'CALIBRE_RESTARTING_FROM_GUI', None) == b'1':
-        time.sleep(2)  # give the parent process time to cleanup and close
     if iswindows and 'CALIBRE_REPAIR_CORRUPTED_DB' in os.environ:
         windows_repair()
         return 0
@@ -516,56 +505,22 @@ def main(args=sys.argv):
         gui_debug = args[1]
         args = ['calibre']
 
-    if iswindows:
-        # Ensure that all ebook editor instances are grouped together in the task
-        # bar. This prevents them from being grouped with viewer process when
-        # launched from within calibre, as both use calibre-parallel.exe
-        set_app_uid(MAIN_APP_UID)
-
     try:
         app, opts, args = init_qt(args)
     except AbortInit:
         return 1
-    try:
-        si = singleinstance(singleinstance_name)
-    except Exception:
-        error_dialog(None, _('Cannot start calibre'), _(
-            'Failed to start calibre, single instance locking failed. Click "Show Details" for more information'),
-                     det_msg=traceback.format_exc(), show=True)
-        return 1
-    if si and opts.shutdown_running_calibre:
-        return 0
+    with SingleInstance(singleinstance_name) as si:
+        if si and opts.shutdown_running_calibre:
+            return 0
+        run_main(app, opts, args, gui_debug, si)
+    if after_quit_actions['restart_after_quit']:
+        restart_after_quit()
+
+
+def run_main(app, opts, args, gui_debug, si):
     if si:
-        try:
-            listener = create_listener()
-        except socket.error:
-            if iswindows or islinux:
-                cant_start(det_msg=traceback.format_exc(), listener_failed=True)
-            if os.path.exists(gui_socket_address()):
-                os.remove(gui_socket_address())
-            try:
-                listener = create_listener()
-            except socket.error:
-                cant_start(det_msg=traceback.format_exc(), listener_failed=True)
-            else:
-                return run_gui(opts, args, listener, app,
-                        gui_debug=gui_debug)
-        else:
-            return run_gui(opts, args, listener, app,
-                    gui_debug=gui_debug)
-    otherinstance = False
-    try:
-        listener = create_listener()
-    except socket.error:  # Good singleinstance is correct (on UNIX)
-        otherinstance = True
-    else:
-        # On windows only singleinstance can be trusted
-        otherinstance = True if iswindows else False
-    if not otherinstance and not opts.shutdown_running_calibre:
-        return run_gui(opts, args, listener, app, gui_debug=gui_debug)
-
+        return run_gui(opts, args, app, gui_debug=gui_debug)
     communicate(opts, args)
-
     return 0
 
 
@@ -579,9 +534,10 @@ if __name__ == '__main__':
         from PyQt5.Qt import QErrorMessage
         logfile = os.path.join(os.path.expanduser('~'), 'calibre.log')
         if os.path.exists(logfile):
-            log = open(logfile).read().decode('utf-8', 'ignore')
+            with open(logfile) as f:
+                log = f.read().decode('utf-8', 'ignore')
             d = QErrorMessage()
             d.showMessage(('<b>Error:</b>%s<br><b>Traceback:</b><br>'
-                '%s<b>Log:</b><br>%s')%(unicode(err),
-                    unicode(tb).replace('\n', '<br>'),
+                '%s<b>Log:</b><br>%s')%(unicode_type(err),
+                    unicode_type(tb).replace('\n', '<br>'),
                     log.replace('\n', '<br>')))
