@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,83 +9,60 @@ __docformat__ = 'restructuredtext en'
 
 '''The main GUI'''
 
-import collections, os, sys, textwrap, time, gc, errno, re
-from threading import Thread
+import apsw
+import collections
+import errno
+import gc
+import os
+import re
+import sys
+import textwrap
+import time
 from collections import OrderedDict
 from io import BytesIO
-
-import apsw
 from PyQt5.Qt import (
-    Qt, QTimer, QAction, QMenu, QIcon, pyqtSignal, QUrl, QFont, QDialog,
-    QApplication, QSystemTrayIcon)
+    QAction, QApplication, QDialog, QFont, QIcon, QMenu, QSystemTrayIcon, Qt, QTimer,
+    QUrl, pyqtSignal
+)
 
-from calibre import prints, force_unicode, detect_ncpus
+from calibre import detect_ncpus, force_unicode, prints
 from calibre.constants import (
-        __appname__, isosx, iswindows, filesystem_encoding, DEBUG, config_dir)
-from calibre.utils.config import prefs, dynamic
-from calibre.utils.ipc.pool import Pool
+    DEBUG, __appname__, config_dir, filesystem_encoding, ismacos, iswindows
+)
+from calibre.customize.ui import available_store_plugins, interface_actions
 from calibre.db.legacy import LibraryDatabase
-from calibre.customize.ui import interface_actions, available_store_plugins
-from calibre.gui2 import (error_dialog, GetMetadata, open_url,
-        gprefs, max_available_height, config, info_dialog, Dispatcher,
-        question_dialog, warning_dialog)
-from calibre.gui2.cover_flow import CoverFlowMixin
+from calibre.gui2 import (
+    Dispatcher, GetMetadata, config, error_dialog, gprefs, info_dialog,
+    max_available_height, open_url, question_dialog, warning_dialog
+)
+from calibre.gui2.auto_add import AutoAdder
 from calibre.gui2.changes import handle_changes
-from calibre.gui2.widgets import ProgressIndicator
-from calibre.gui2.update import UpdateMixin
-from calibre.gui2.main_window import MainWindow
-from calibre.gui2.layout import MainWindowMixin
+from calibre.gui2.cover_flow import CoverFlowMixin
+from calibre.gui2.dbus_export.widgets import factory
 from calibre.gui2.device import DeviceMixin
-from calibre.gui2.email import EmailMixin
+from calibre.gui2.dialogs.message_box import JobError
 from calibre.gui2.ebook_download import EbookDownloadMixin
-from calibre.gui2.jobs import JobManager, JobsDialog, JobsButton
-from calibre.gui2.init import LibraryViewMixin, LayoutMixin
-from calibre.gui2.search_box import SearchBoxMixin, SavedSearchBoxMixin
+from calibre.gui2.email import EmailMixin
+from calibre.gui2.init import LayoutMixin, LibraryViewMixin
+from calibre.gui2.job_indicator import Pointer
+from calibre.gui2.jobs import JobManager, JobsButton, JobsDialog
+from calibre.gui2.keyboard import Manager
+from calibre.gui2.layout import MainWindowMixin
+from calibre.gui2.listener import Listener
+from calibre.gui2.main_window import MainWindow
+from calibre.gui2.open_with import register_keyboard_shortcuts
+from calibre.gui2.proceed import ProceedQuestion
+from calibre.gui2.search_box import SavedSearchBoxMixin, SearchBoxMixin
 from calibre.gui2.search_restriction_mixin import SearchRestrictionMixin
 from calibre.gui2.tag_browser.ui import TagBrowserMixin
-from calibre.gui2.keyboard import Manager
-from calibre.gui2.auto_add import AutoAdder
-from calibre.gui2.proceed import ProceedQuestion
-from calibre.gui2.dialogs.message_box import JobError
-from calibre.gui2.job_indicator import Pointer
-from calibre.gui2.dbus_export.widgets import factory
-from calibre.gui2.open_with import register_keyboard_shortcuts
+from calibre.gui2.update import UpdateMixin
+from calibre.gui2.widgets import ProgressIndicator
 from calibre.library import current_library_name
 from calibre.srv.library_broker import GuiLibraryBroker
-from polyglot.builtins import unicode_type, string_or_bytes
-from polyglot.queue import Queue, Empty
-
-
-class Listener(Thread):  # {{{
-
-    def __init__(self, listener):
-        Thread.__init__(self)
-        self.daemon = True
-        self.listener, self.queue = listener, Queue()
-        self._run = True
-        self.start()
-
-    def run(self):
-        if self.listener is None:
-            return
-        while self._run:
-            try:
-                conn = self.listener.accept()
-                msg = conn.recv()
-                self.queue.put(msg)
-            except:
-                continue
-
-    def close(self):
-        self._run = False
-        try:
-            if self.listener is not None:
-                self.listener.close()
-        except:
-            import traceback
-            traceback.print_exc()
-
-# }}}
+from calibre.utils.config import dynamic, prefs
+from calibre.utils.ipc.pool import Pool
+from polyglot.builtins import string_or_bytes, unicode_type
+from polyglot.queue import Empty, Queue
 
 
 def get_gui():
@@ -93,11 +70,11 @@ def get_gui():
 
 
 def add_quick_start_guide(library_view, refresh_cover_browser=None):
-    from calibre.ebooks.metadata.meta import get_metadata
     from calibre.ebooks.covers import calibre_cover2
-    from calibre.utils.zipfile import safe_replace
-    from calibre.utils.localization import get_lang, canonicalize_lang
+    from calibre.ebooks.metadata.meta import get_metadata
     from calibre.ptempfile import PersistentTemporaryFile
+    from calibre.utils.localization import canonicalize_lang, get_lang
+    from calibre.utils.zipfile import safe_replace
     l = canonicalize_lang(get_lang()) or 'eng'
     gprefs['quick_start_guide_added'] = True
     imgbuf = BytesIO(calibre_cover2(_('Quick Start Guide'), ''))
@@ -158,7 +135,11 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             except:
                 # Ignore errors in loading user supplied plugins
                 import traceback
-                traceback.print_exc()
+                try:
+                    traceback.print_exc()
+                except Exception:
+                    if action.plugin_path:
+                        print('Failed to load Interface Action plugin:', action.plugin_path, file=sys.stderr)
                 if action.plugin_path is None:
                     raise
                 continue
@@ -215,7 +196,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         else:
             stmap[st.name] = st
 
-    def initialize(self, library_path, db, listener, actions, show_gui=True):
+    def initialize(self, library_path, db, actions, show_gui=True):
         opts = self.opts
         self.preferences_action, self.quit_action = actions
         self.library_path = library_path
@@ -226,10 +207,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         t.setInterval(1000), t.timeout.connect(self.handle_changes_from_server_debounced), t.setSingleShot(True)
         self._spare_pool = None
         self.must_restart_before_config = False
-        self.listener = Listener(listener)
-        self.check_messages_timer = QTimer()
-        self.check_messages_timer.timeout.connect(self.another_instance_wants_to_talk)
-        self.check_messages_timer.start(1000)
 
         for ac in self.iactions.values():
             try:
@@ -271,7 +248,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self.system_tray_icon = factory(app_id='com.calibre-ebook.gui').create_system_tray_icon(parent=self, title='calibre')
         if self.system_tray_icon is not None:
             self.system_tray_icon.setIcon(QIcon(I('lt.png', allow_user_override=False)))
-            if not (iswindows or isosx):
+            if not (iswindows or ismacos):
                 self.system_tray_icon.setIcon(QIcon.fromTheme('calibre-tray', self.system_tray_icon.icon()))
             self.system_tray_icon.setToolTip(self.jobs_button.tray_tooltip())
             self.system_tray_icon.setVisible(True)
@@ -318,7 +295,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.ctrl_esc_action = QAction(self)
         self.addAction(self.ctrl_esc_action)
         self.keyboard.register_shortcut('clear virtual library',
-                _('Clear the virtual library'), default_keys=('Ctrl+Esc',),
+                _('Clear the Virtual library'), default_keys=('Ctrl+Esc',),
                 action=self.ctrl_esc_action)
         self.ctrl_esc_action.triggered.connect(self.ctrl_esc)
 
@@ -373,8 +350,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.library_view.model().database_changed.connect(self.populate_tb_manage_menu, type=Qt.QueuedConnection)
 
         # ######################## Search Restriction ##########################
-        if db.prefs['virtual_lib_on_startup']:
-            self.apply_virtual_library(db.prefs['virtual_lib_on_startup'])
+        if db.new_api.pref('virtual_lib_on_startup'):
+            self.apply_virtual_library(db.new_api.pref('virtual_lib_on_startup'))
         self.rebuild_vl_tabs()
 
         # ########################## Cover Flow ################################
@@ -424,14 +401,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self.hide_windows()
         self.auto_adder = AutoAdder(gprefs['auto_add_path'], self)
 
-        # Now that the gui is initialized we can restore the quickview state
-        # The same thing will be true for any action-based operation with a
-        # layout button
-        from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
-        qv = get_quickview_action_plugin()
-        if qv:
-            qv.qv_button.restore_state()
-        self.save_layout_state()
+        self.listener = Listener(parent=self)
+        self.listener.message_received.connect(self.message_from_another_instance)
+        QTimer.singleShot(0, self.listener.start_listening)
 
         # Collect cycles now
         gc.collect()
@@ -444,6 +416,22 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         QTimer.singleShot(1, self.start_smartdevice)
         QTimer.singleShot(100, self.update_toggle_to_tray_action)
 
+        # Once the gui is initialized we can restore the quickview state
+        # The same thing will be true for any action-based operation with a
+        # layout button. We need to let a book be selected in the book list
+        # before initializing quickview, so run it after an event loop tick
+        QTimer.singleShot(0, self.start_quickview)
+
+    def start_quickview(self):
+        from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+        qv = get_quickview_action_plugin()
+        if qv:
+            if DEBUG:
+                prints('Starting QuickView')
+            qv.qv_button.restore_state()
+        self.save_layout_state()
+        self.library_view.setFocus(Qt.OtherFocusReason)
+
     def show_gui_debug_msg(self):
         info_dialog(self, _('Debug mode'), '<p>' +
                 _('You have started calibre in debug mode. After you '
@@ -613,17 +601,131 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.tags_view.recount()
 
     def handle_cli_args(self, args):
+        from urllib.parse import unquote, urlparse, parse_qs
         if isinstance(args, string_or_bytes):
             args = [args]
-        files = [os.path.abspath(p) for p in args if not os.path.isdir(p) and os.access(p, os.R_OK)]
+        files, urls = [], []
+        for p in args:
+            if p.startswith('calibre://'):
+                try:
+                    purl = urlparse(p)
+                    if purl.scheme == 'calibre':
+                        action = purl.netloc
+                        path = unquote(purl.path)
+                        query = parse_qs(unquote(purl.query))
+                        urls.append((action, path, query))
+                except Exception:
+                    prints('Ignoring malformed URL:', p, file=sys.stderr)
+                    continue
+            elif p.startswith('file://'):
+                try:
+                    purl = urlparse(p)
+                    if purl.scheme == 'file':
+                        path = unquote(purl.path)
+                        a = os.path.abspath(path)
+                        if not os.path.isdir(a) and os.access(a, os.R_OK):
+                            files.append(a)
+                except Exception:
+                    prints('Ignoring malformed URL:', p, file=sys.stderr)
+                    continue
+            else:
+                a = os.path.abspath(p)
+                if not os.path.isdir(a) and os.access(a, os.R_OK):
+                    files.append(a)
         if files:
             self.iactions['Add Books'].add_filesystem_book(files)
+        if urls:
+            def doit():
+                for action, path, query in urls:
+                    self.handle_url_action(action, path, query)
+            QTimer.singleShot(10, doit)
 
-    def another_instance_wants_to_talk(self):
-        try:
-            msg = self.listener.queue.get_nowait()
-        except Empty:
-            return
+    def handle_url_action(self, action, path, query):
+        import posixpath
+
+        def decode_library_id(x):
+            if x == '_':
+                return getattr(self.current_db.new_api, 'server_library_id', None) or '_'
+            if x.startswith('_hex_-'):
+                return bytes.fromhex(x[6:]).decode('utf-8')
+            return x
+
+        if action == 'switch-library':
+            library_id = decode_library_id(posixpath.basename(path))
+            library_path = self.library_broker.path_for_library_id(library_id)
+            if library_path is not None and library_id != getattr(self.current_db.new_api, 'server_library_id', None):
+                self.library_moved(library_path)
+        elif action == 'show-book':
+            parts = tuple(filter(None, path.split('/')))
+            if len(parts) != 2:
+                return
+            library_id, book_id = parts
+            library_id = decode_library_id(library_id)
+            try:
+                book_id = int(book_id)
+            except Exception:
+                prints('Ignoring invalid book id', book_id, file=sys.stderr)
+                return
+            library_path = self.library_broker.path_for_library_id(library_id)
+            if library_path is None:
+                return
+
+            def doit():
+                self.library_view.select_rows((book_id,))
+
+            if library_id != getattr(self.current_db.new_api, 'server_library_id', None):
+                self.library_moved(library_path)
+                QTimer.singleShot(0, doit)
+            else:
+                doit()
+
+        elif action == 'view-book':
+            parts = tuple(filter(None, path.split('/')))
+            if len(parts) != 3:
+                return
+            library_id, book_id, fmt = parts
+            library_id = decode_library_id(library_id)
+            try:
+                book_id = int(book_id)
+            except Exception:
+                prints('Ignoring invalid book id', book_id, file=sys.stderr)
+                return
+            library_path = self.library_broker.path_for_library_id(library_id)
+            if library_path is None:
+                return
+            view = self.iactions['View']
+
+            def doit():
+                at = query.get('open_at') or None
+                if at:
+                    at = at[0]
+                view.view_format_by_id(book_id, fmt.upper(), open_at=at)
+
+            if library_id != getattr(self.current_db.new_api, 'server_library_id', None):
+                self.library_moved(library_path)
+                QTimer.singleShot(0, doit)
+            else:
+                doit()
+
+        elif action == 'search':
+            parts = tuple(filter(None, path.split('/')))
+            if len(parts) != 1:
+                return
+            library_id = decode_library_id(parts[0])
+            library_path = self.library_broker.path_for_library_id(library_id)
+            if library_path is None:
+                return
+            sq = query.get('eq')
+            if sq:
+                sq = bytes.fromhex(sq[0]).decode('utf-8')
+            else:
+                sq = query.get('q')
+                if sq:
+                    sq = sq[0]
+            sq = sq or ''
+            self.search.set_search_string(sq)
+
+    def message_from_another_instance(self, msg):
         if isinstance(msg, bytes):
             msg = msg.decode('utf-8', 'replace')
         if msg.startswith('launched:'):
@@ -644,10 +746,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self.show_windows()
             self.raise_()
             self.activateWindow()
-        elif msg.startswith('refreshdb:'):
-            m = self.library_view.model()
-            m.db.new_api.reload_from_db()
-            self.refresh_all()
         elif msg.startswith('shutdown:'):
             self.quit(confirm_quit=False)
         elif msg.startswith('bookedited:'):
@@ -707,7 +805,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             try:
                 olddb = self.library_view.model().db
                 if copy_structure:
-                    default_prefs = olddb.prefs
+                    default_prefs = dict(olddb.prefs)
             except:
                 olddb = None
             if copy_structure and olddb is not None and default_prefs is not None:
@@ -728,7 +826,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                             det_msg=traceback.format_exc()
                             )
                     if repair:
-                        from calibre.gui2.dialogs.restore_library import repair_library_at
+                        from calibre.gui2.dialogs.restore_library import (
+                            repair_library_at
+                        )
                         if repair_library_at(newloc, parent=self):
                             db = LibraryDatabase(newloc, default_prefs=default_prefs)
                         else:
@@ -752,8 +852,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self.set_window_title()
             self.apply_named_search_restriction('')  # reset restriction to null
             self.saved_searches_changed(recount=False)  # reload the search restrictions combo box
-            if db.prefs['virtual_lib_on_startup']:
-                self.apply_virtual_library(db.prefs['virtual_lib_on_startup'])
+            if db.new_api.pref('virtual_lib_on_startup'):
+                self.apply_virtual_library(db.new_api.pref('virtual_lib_on_startup'))
             self.rebuild_vl_tabs()
             for action in self.iactions.values():
                 action.library_changed(db)
@@ -911,7 +1011,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def read_settings(self):
         geometry = config['main_window_geometry']
         if geometry is not None:
-            self.restoreGeometry(geometry)
+            QApplication.instance().safe_restore_geometry(self, geometry)
         self.read_layout_settings()
 
     def write_settings(self):
@@ -930,7 +1030,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         try:
             self.shutdown()
         except:
-            pass
+            import traceback
+            traceback.print_exc()
         self.restart_after_quit = restart
         self.debug_on_restart = debug_on_restart
         if self.system_tray_icon is not None and self.restart_after_quit:
@@ -994,11 +1095,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             db.commit_dirty_cache()
             db.prefs.write_serialized(prefs['library_path'])
         for action in self.iactions.values():
-            if not action.shutting_down():
-                return
+            action.shutting_down()
         if write_settings:
             self.write_settings()
-        self.check_messages_timer.stop()
         if getattr(self, 'update_checker', None):
             self.update_checker.shutdown()
         self.listener.close()
@@ -1063,7 +1162,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             return
         self.write_settings()
         if self.system_tray_icon is not None and self.system_tray_icon.isVisible():
-            if not dynamic['systray_msg'] and not isosx:
+            if not dynamic['systray_msg'] and not ismacos:
                 info_dialog(self, 'calibre', 'calibre '+
                         _('will keep running in the system tray. To close it, '
                         'choose <b>Quit</b> in the context menu of the '
