@@ -1,42 +1,41 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, traceback, re
-from contextlib import closing
+import os
+import re
+import traceback
+from contextlib import closing, suppress
+from PyQt5.Qt import (
+    QAbstractListModel, QDir, QIcon, QItemSelection, QItemSelectionModel, Qt,
+    QWizard, QWizardPage, pyqtSignal
+)
 
-
-from PyQt5.Qt import (QWizard, QWizardPage, QPixmap, Qt, QAbstractListModel,
-    QItemSelectionModel, pyqtSignal, QItemSelection, QDir)
 from calibre import __appname__
-from calibre.constants import (filesystem_encoding, iswindows, plugins,
-        isportable)
-from calibre.gui2.wizard.send_email import smtp_prefs
+from calibre.constants import filesystem_encoding, isportable, iswindows
+from calibre.gui2 import choose_dir, error_dialog
 from calibre.gui2.wizard.device_ui import Ui_WizardPage as DeviceUI
-from calibre.gui2.wizard.library_ui import Ui_WizardPage as LibraryUI
 from calibre.gui2.wizard.finish_ui import Ui_WizardPage as FinishUI
 from calibre.gui2.wizard.kindle_ui import Ui_WizardPage as KindleUI
+from calibre.gui2.wizard.library_ui import Ui_WizardPage as LibraryUI
+from calibre.gui2.wizard.send_email import smtp_prefs
 from calibre.gui2.wizard.stanza_ui import Ui_WizardPage as StanzaUI
-from calibre.gui2 import min_available_height, available_width
-from calibre.utils.localization import localize_user_manual_link
-
 from calibre.utils.config import dynamic, prefs
-from calibre.gui2 import choose_dir, error_dialog
-
-if iswindows:
-    winutil = plugins['winutil'][0]
+from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import iteritems, map, unicode_type
 
 # Devices {{{
 
+
 class Device(object):
 
     output_profile = 'generic_eink'
     output_format = 'EPUB'
-    name = 'Generic e-ink device'
+    name = _('Generic e-ink device')
     manufacturer = 'Generic'
     id = 'default'
     supports_color = False
@@ -64,27 +63,31 @@ class Device(object):
             recs['dont_grayscale'] = True
             save_defaults('comic_input', recs)
 
+
 class Smartphone(Device):
 
     id = 'smartphone'
-    name = 'Smartphone'
+    name = _('Smartphone')
     supports_color = True
 
+
 class Tablet(Device):
 
     id = 'tablet'
-    name = 'iPad like tablet'
+    name = _('iPad like tablet')
     output_profile = 'tablet'
     supports_color = True
 
+
 class Kindle(Device):
 
     output_profile = 'kindle'
     output_format  = 'MOBI'
-    name = 'Kindle Touch/1-4'
+    name = _('Kindle Basic (all models)')
     manufacturer = 'Amazon'
     id = 'kindle'
 
+
 class JetBook(Device):
 
     output_profile = 'jetbook5'
@@ -93,6 +96,7 @@ class JetBook(Device):
     manufacturer = 'Ectaco'
     id = 'jetbook'
 
+
 class JetBookMini(Device):
 
     output_profile = 'jetbook5'
@@ -101,6 +105,7 @@ class JetBookMini(Device):
     manufacturer = 'Ectaco'
     id = 'jetbookmini'
 
+
 class KindleDX(Kindle):
 
     output_profile = 'kindle_dx'
@@ -108,42 +113,49 @@ class KindleDX(Kindle):
     name = 'Kindle DX'
     id = 'kindledx'
 
+
 class KindleFire(KindleDX):
-    name = 'Kindle Fire and Fire HD'
+    name = _('{0} and {1}').format('Kindle Fire', 'Fire HD')
     id = 'kindle_fire'
     output_profile = 'kindle_fire'
     supports_color = True
 
+
 class KindlePW(Kindle):
     name = 'Kindle PaperWhite'
     id = 'kindle_pw'
-    output_profile = 'kindle_pw'
+    output_profile = 'kindle_pw3'
+
 
 class KindleVoyage(Kindle):
     name = 'Kindle Voyage/Oasis'
     id = 'kindle_voyage'
     output_profile = 'kindle_voyage'
 
+
 class Sony505(Device):
 
     output_profile = 'sony'
-    name = 'All other SONY devices'
+    name = _('All other SONY devices')
     output_format = 'EPUB'
     manufacturer = 'SONY'
     id = 'prs505'
 
+
 class Kobo(Device):
-    name = 'Kobo and Kobo Touch Readers'
+    name = _('Kobo and Kobo Touch Readers')
     manufacturer = 'Kobo'
     output_profile = 'kobo'
     output_format = 'EPUB'
     id = 'kobo'
 
+
 class KoboVox(Kobo):
-    name = 'Kobo Vox, Aura and Glo families'
+    name = _('Kobo Vox, Aura and Glo families')
     output_profile = 'tablet'
     id = 'kobo_vox'
 
+
 class Booq(Device):
     name = 'bq Classic'
     manufacturer = 'Booq'
@@ -151,6 +163,7 @@ class Booq(Device):
     output_format = 'EPUB'
     id = 'booq'
 
+
 class TheBook(Device):
     name = 'The Book'
     manufacturer = 'Augen'
@@ -158,55 +171,74 @@ class TheBook(Device):
     output_format = 'EPUB'
     id = 'thebook'
 
+
 class Avant(Booq):
     name = 'bq Avant'
 
+
 class AvantXL(Booq):
     name = 'bq Avant XL'
     output_profile = 'ipad'
 
+
 class BooqPocketPlus(Booq):
     name = 'bq Pocket Plus'
     output_profile = 'sony300'
 
+
 class BooqCervantes(Booq):
     name = 'bq Cervantes'
 
+
+class BOOX(Device):
+    name = _('BOOX MAX, N96, i86, C67ML, M96, etc.')
+    manufacturer = 'Onyx'
+    output_profile = 'generic_eink_hd'
+    output_format = 'EPUB'
+    id = 'boox_eink'
+
+
 class Sony300(Sony505):
 
     name = 'SONY Reader Pocket Edition'
     id = 'prs300'
     output_profile = 'sony300'
 
+
 class Sony900(Sony505):
 
     name = 'SONY Reader Daily Edition'
     id = 'prs900'
     output_profile = 'sony900'
 
+
 class SonyT3(Sony505):
 
     name = 'SONY Reader T3'
     id = 'prst3'
     output_profile = 'sonyt3'
 
+
 class Nook(Sony505):
     id = 'nook'
-    name = 'Nook and Nook Simple Reader'
+    name = _('{0} and {1}').format('Nook', 'Nook Simple Reader')
     manufacturer = 'Barnes & Noble'
     output_profile = 'nook'
 
+
 class NookColor(Nook):
     id = 'nook_color'
     name = 'Nook Color'
     output_profile = 'nook_color'
     supports_color = True
 
+
 class NookTablet(NookColor):
     id = 'nook_tablet'
     name = 'Nook Tablet/HD'
     output_profile = 'nook_hd_plus'
 
+
 class CybookG3(Device):
 
     name = 'Cybook Gen 3'
@@ -215,6 +247,7 @@ class CybookG3(Device):
     manufacturer = 'Bookeen'
     id = 'cybookg3'
 
+
 class CybookOpus(CybookG3):
 
     name = 'Cybook Opus'
@@ -222,29 +255,41 @@ class CybookOpus(CybookG3):
     output_profile = 'cybook_opus'
     id = 'cybook_opus'
 
+
 class CybookOrizon(CybookOpus):
 
     name = 'Cybook Orizon'
     id = 'cybook_orizon'
 
+
 class CybookOdyssey(CybookOpus):
 
     name = 'Cybook Odyssey'
     id = 'cybook_odyssey'
 
+
 class CybookMuse(CybookOpus):
 
     name = 'Cybook Muse'
     id = 'cybook_muse'
     output_profile = 'tablet'
 
+
+class BookeenDiva(CybookOpus):
+
+    name = 'Bookeen Diva HD'
+    id = 'bookeen_diva'
+    output_profile = 'tablet'
+
+
 class PocketBook360(CybookOpus):
 
     manufacturer = 'PocketBook'
-    name = 'PocketBook 360 and newer models'
+    name = _('PocketBook 360 and newer models')
     id = 'pocketbook360'
     output_profile = 'cybook_opus'
 
+
 class PocketBook(CybookG3):
 
     manufacturer = 'PocketBook'
@@ -252,12 +297,14 @@ class PocketBook(CybookG3):
     id = 'pocketbook'
     output_profile = 'cybookg3'
 
+
 class PocketBook900(PocketBook):
 
     name = 'PocketBook 900'
     id = 'pocketbook900'
     output_profile = 'pocketbook_900'
 
+
 class PocketBookPro912(PocketBook):
 
     name = 'PocketBook Pro 912'
@@ -274,9 +321,10 @@ class iPhone(Device):
     supports_color = True
     output_profile = 'ipad3'
 
+
 class Android(Device):
 
-    name = 'Android phone'
+    name = _('Android phone')
     output_format = 'EPUB'
     manufacturer = 'Android'
     id = 'android'
@@ -290,15 +338,17 @@ class Android(Device):
             if hasattr(plugin, 'configure_for_generic_epub_app'):
                 plugin.configure_for_generic_epub_app()
 
+
 class AndroidTablet(Android):
 
-    name = 'Android tablet'
+    name = _('Android tablet')
     id = 'android_tablet'
     output_profile = 'tablet'
 
+
 class AndroidPhoneWithKindle(Android):
 
-    name = 'Android phone with Kindle reader'
+    name = _('Android phone with Kindle reader')
     output_format = 'MOBI'
     id = 'android_phone_with_kindle'
     output_profile = 'kindle'
@@ -311,12 +361,14 @@ class AndroidPhoneWithKindle(Android):
             if hasattr(plugin, 'configure_for_kindle_app'):
                 plugin.configure_for_kindle_app()
 
+
 class AndroidTabletWithKindle(AndroidPhoneWithKindle):
 
-    name = 'Android tablet with Kindle reader'
+    name = _('Android tablet with Kindle reader')
     id = 'android_tablet_with_kindle'
     output_profile = 'kindle_fire'
 
+
 class HanlinV3(Device):
 
     name = 'Hanlin V3'
@@ -325,30 +377,35 @@ class HanlinV3(Device):
     manufacturer = 'Jinke'
     id = 'hanlinv3'
 
+
 class HanlinV5(HanlinV3):
 
     name = 'Hanlin V5'
     output_profile = 'hanlinv5'
     id = 'hanlinv5'
 
+
 class BeBook(HanlinV3):
 
     name = 'BeBook'
     manufacturer = 'BeBook'
     id = 'bebook'
 
+
 class BeBookMini(HanlinV5):
 
     name = 'BeBook Mini'
     manufacturer = 'BeBook'
     id = 'bebook_mini'
 
+
 class EZReader(HanlinV3):
 
     name = 'EZReader'
     manufacturer = 'Astak'
     id = 'ezreader'
 
+
 class EZReaderPP(HanlinV5):
 
     name = 'EZReader Pocket Pro'
@@ -357,22 +414,26 @@ class EZReaderPP(HanlinV5):
 
 # }}}
 
+
 def get_devices():
     for x in globals().values():
         if isinstance(x, type) and issubclass(x, Device):
             yield x
 
+
 def get_manufacturers():
-    mans = set([])
+    mans = set()
     for x in get_devices():
         mans.add(x.manufacturer)
     if Device.manufacturer in mans:
         mans.remove(Device.manufacturer)
     return [Device.manufacturer] + sorted(mans)
 
+
 def get_devices_of(manufacturer):
     ans = [d for d in get_devices() if d.manufacturer == manufacturer]
-    return sorted(ans, cmp=lambda x,y:cmp(x.name, y.name))
+    return sorted(ans, key=lambda x: x.name)
+
 
 class ManufacturerModel(QAbstractListModel):
 
@@ -388,7 +449,10 @@ class ManufacturerModel(QAbstractListModel):
 
     def data(self, index, role):
         if role == Qt.DisplayRole:
-            return (self.manufacturers[index.row()])
+            ans = self.manufacturers[index.row()]
+            if ans == Device.manufacturer:
+                ans = _('Generic')
+            return ans
         if role == Qt.UserRole:
             return self.manufacturers[index.row()]
         return None
@@ -398,6 +462,7 @@ class ManufacturerModel(QAbstractListModel):
             if x == man:
                 return self.index(i)
 
+
 class DeviceModel(QAbstractListModel):
 
     def __init__(self, manufacturer):
@@ -422,6 +487,7 @@ class DeviceModel(QAbstractListModel):
             if device is dev:
                 return self.index(i)
 
+
 class KindlePage(QWizardPage, KindleUI):
 
     ID = 3
@@ -434,7 +500,7 @@ class KindlePage(QWizardPage, KindleUI):
         opts = smtp_prefs().parse()
         accs = []
         has_default = False
-        for x, ac in opts.accounts.iteritems():
+        for x, ac in iteritems(opts.accounts):
             default = ac[2]
             if x.strip().endswith('@kindle.com'):
                 accs.append((x, default))
@@ -444,15 +510,16 @@ class KindlePage(QWizardPage, KindleUI):
             accs = [x for x in accs if x[1]]
         if accs:
             self.to_address.setText(accs[0][0])
+
         def x():
-            t = unicode(self.to_address.text())
+            t = unicode_type(self.to_address.text())
             if t.strip():
                 return t.strip()
 
         self.send_email_widget.initialize(x)
 
     def commit(self):
-        x = unicode(self.to_address.text()).strip()
+        x = unicode_type(self.to_address.text()).strip()
         parts = x.split('@')
 
         if (len(parts) >= 2 and parts[0] and self.send_email_widget.set_email_settings(True)):
@@ -473,6 +540,7 @@ class KindlePage(QWizardPage, KindleUI):
         if hasattr(self, 'send_email_widget'):
             self.send_email_widget.retranslateUi(self.send_email_widget)
 
+
 class StanzaPage(QWizardPage, StanzaUI):
 
     ID = 5
@@ -500,9 +568,8 @@ class StanzaPage(QWizardPage, StanzaUI):
     def commit(self):
         p = self.set_port()
         if p is not None:
-            from calibre.library.server import server_config
-            c = server_config()
-            c.set('port', p)
+            from calibre.srv.opts import change_settings
+            change_settings(port=p)
 
     def set_port(self, *args):
         if not self.content_server.isChecked():
@@ -513,8 +580,8 @@ class StanzaPage(QWizardPage, StanzaUI):
             for p in range(8080, 8100):
                 try:
                     s.bind(('0.0.0.0', p))
-                    t = unicode(self.instructions.text())
-                    t = re.sub(r':\d+', ':'+str(p), t)
+                    t = unicode_type(self.instructions.text())
+                    t = re.sub(r':\d+', ':'+unicode_type(p), t)
                     self.instructions.setText(t)
                     return p
                 except:
@@ -533,7 +600,7 @@ class DevicePage(QWizardPage, DeviceUI):
 
     def initializePage(self):
         self.label.setText(_('Choose your e-book device. If your device is'
-            ' not in the list, choose a "%s" device.')%Device.manufacturer)
+            ' not in the list, choose a "Generic" device.'))
         self.man_model = ManufacturerModel()
         self.manufacturer_view.setModel(self.man_model)
         previous = dynamic.get('welcome_wizard_device', False)
@@ -580,6 +647,7 @@ class DevicePage(QWizardPage, DeviceUI):
             return StanzaPage.ID
         return FinishPage.ID
 
+
 class LibraryPage(QWizardPage, LibraryUI):
 
     ID = 1
@@ -587,25 +655,45 @@ class LibraryPage(QWizardPage, LibraryUI):
 
     def __init__(self):
         QWizardPage.__init__(self)
+        self.made_dirs = []
+        self.initial_library_location = None
         self.setupUi(self)
         self.registerField('library_location', self.location)
         self.button_change.clicked.connect(self.change)
         self.init_languages()
         self.language.currentIndexChanged[int].connect(self.change_language)
         self.location.textChanged.connect(self.location_text_changed)
+        self.set_move_lib_label_text()
+
+    def makedirs(self, x):
+        self.made_dirs.append(x)
+        os.makedirs(x)
 
     def location_text_changed(self, newtext):
         self.completeChanged.emit()
 
+    def set_move_lib_label_text(self):
+        self.move_lib_label.setText(_(
+            'If you are moving calibre from an old computer to a new one,'
+            ' please read <a href="{0}">the instructions</a>.').format(
+                localize_user_manual_link(
+        'https://manual.calibre-ebook.com/faq.html#how-do-i-move-my-calibre-data-from-one-computer-to-another')))
+
+    def retranslateUi(self, widget):
+        LibraryUI.retranslateUi(self, widget)
+        self.set_move_lib_label_text()
+
     def init_languages(self):
         self.language.blockSignals(True)
         self.language.clear()
-        from calibre.utils.localization import (available_translations,
-            get_language, get_lang, get_lc_messages_path)
+        from calibre.utils.localization import (
+            available_translations, get_lang, get_language, get_lc_messages_path
+        )
         lang = get_lang()
         lang = get_lc_messages_path(lang) if lang else lang
         if lang is None or lang not in available_translations():
             lang = 'en'
+
         def get_esc_lang(l):
             if l == 'en':
                 return 'English'
@@ -616,19 +704,19 @@ class LibraryPage(QWizardPage, LibraryUI):
                  if l != lang]
         if lang != 'en':
             items.append(('en', get_esc_lang('en')))
-        items.sort(cmp=lambda x, y: cmp(x[1], y[1]))
+        items.sort(key=lambda x: x[1])
         for item in items:
             self.language.addItem(item[1], (item[0]))
         self.language.blockSignals(False)
-        prefs['language'] = str(self.language.itemData(self.language.currentIndex()) or '')
+        prefs['language'] = unicode_type(self.language.itemData(self.language.currentIndex()) or '')
 
     def change_language(self, idx):
-        prefs['language'] = str(self.language.itemData(self.language.currentIndex()) or '')
-        import __builtin__
-        __builtin__.__dict__['_'] = lambda(x): x
-        from calibre.utils.localization import set_translators
-        from calibre.gui2 import qt_app
+        prefs['language'] = unicode_type(self.language.itemData(self.language.currentIndex()) or '')
+        from polyglot.builtins import builtins
+        builtins.__dict__['_'] = lambda x: x
         from calibre.ebooks.metadata.book.base import reset_field_metadata
+        from calibre.gui2 import qt_app
+        from calibre.utils.localization import set_translators
         set_translators()
         qt_app.load_translations()
         self.retranslate.emit()
@@ -646,6 +734,9 @@ class LibraryPage(QWizardPage, LibraryUI):
                 enable_plugin(name)
         except:
             pass
+        lp = self.location.text()
+        if lp == self.initial_library_location:
+            self.set_initial_library_location()
 
     def is_library_dir_suitable(self, x):
         from calibre.db.legacy import LibraryDatabase
@@ -655,7 +746,7 @@ class LibraryPage(QWizardPage, LibraryUI):
             return False
 
     def validatePage(self):
-        newloc = unicode(self.location.text())
+        newloc = unicode_type(self.location.text())
         if not self.is_library_dir_suitable(newloc):
             self.show_library_dir_error(newloc)
             return False
@@ -666,15 +757,14 @@ class LibraryPage(QWizardPage, LibraryUI):
         x = choose_dir(self, 'database location dialog',
                          _('Select location for books'))
         if x:
-            if (iswindows and len(x) >
-                    LibraryDatabase.WINDOWS_LIBRARY_PATH_LIMIT):
+            if (iswindows and len(x) > LibraryDatabase.WINDOWS_LIBRARY_PATH_LIMIT):
                 return error_dialog(self, _('Too long'),
                     _('Path to library too long. Must be less than'
                     ' %d characters.')%(LibraryDatabase.WINDOWS_LIBRARY_PATH_LIMIT),
                     show=True)
             if not os.path.exists(x):
                 try:
-                    os.makedirs(x)
+                    self.makedirs(x)
                 except:
                     return error_dialog(self, _('Bad location'),
                             _('Failed to create a folder at %s')%x,
@@ -686,38 +776,41 @@ class LibraryPage(QWizardPage, LibraryUI):
                 self.show_library_dir_error(x)
 
     def show_library_dir_error(self, x):
-        if not isinstance(x, unicode):
+        if not isinstance(x, unicode_type):
             try:
                 x = x.decode(filesystem_encoding)
             except:
-                x = unicode(repr(x))
+                x = unicode_type(repr(x))
         error_dialog(self, _('Bad location'),
             _('You must choose an empty folder for '
                 'the calibre library. %s is not empty.')%x, show=True)
 
-    def initializePage(self):
+    def set_initial_library_location(self):
         lp = prefs['library_path']
         self.default_library_name = None
         if not lp:
             fname = _('Calibre Library')
-            base = os.path.expanduser(u'~')
-            if iswindows:
-                try:
-                    x = winutil.special_folder_path(winutil.CSIDL_PERSONAL)
-                except ValueError:
-                    x = QDir.homePath().replace('/', os.sep)
-                if x and os.access(x, os.W_OK):
-                    base = x
+            try:
+                base = os.path.expanduser('~')
+            except ValueError:
+                base = QDir.homePath().replace('/', os.sep)
 
             lp = os.path.join(base, fname)
             self.default_library_name = lp
             if not os.path.exists(lp):
                 try:
-                    os.makedirs(lp)
+                    self.makedirs(lp)
                 except:
                     traceback.print_exc()
-                    lp = os.path.expanduser(u'~')
+                    try:
+                        lp = os.path.expanduser('~')
+                    except ValueError:
+                        lp = QDir.homePath().replace('/', os.sep)
         self.location.setText(lp)
+        self.initial_library_location = lp
+
+    def initializePage(self):
+        self.set_initial_library_location()
         # Hide the library location settings if we are a portable install
         for x in ('location', 'button_change', 'libloc_label1',
                 'libloc_label2'):
@@ -725,7 +818,7 @@ class LibraryPage(QWizardPage, LibraryUI):
 
     def isComplete(self):
         try:
-            lp = unicode(self.location.text())
+            lp = unicode_type(self.location.text())
             ans = bool(lp) and os.path.exists(lp) and os.path.isdir(lp) and os.access(lp,
                     os.W_OK)
         except:
@@ -733,13 +826,17 @@ class LibraryPage(QWizardPage, LibraryUI):
         return ans
 
     def commit(self):
-        newloc = unicode(self.location.text())
+        newloc = unicode_type(self.location.text())
         try:
             dln = self.default_library_name
             if (dln and os.path.exists(dln) and not os.listdir(dln) and newloc != dln):
                 os.rmdir(dln)
         except Exception:
             pass
+        # dont leave behind any empty dirs
+        for x in self.made_dirs:
+            with suppress(OSError):
+                os.rmdir(x)
         if not os.path.exists(newloc):
             os.makedirs(newloc)
         prefs['library_path'] = newloc
@@ -747,6 +844,7 @@ class LibraryPage(QWizardPage, LibraryUI):
     def nextId(self):
         return DevicePage.ID
 
+
 class FinishPage(QWizardPage, FinishUI):
 
     ID = 4
@@ -777,22 +875,17 @@ class Wizard(QWizard):
     }
     # The latter is simply to mark the texts for translation
     if False:
-            _('&Next >')
-            _('< &Back')
-            _('Cancel')
-            _('&Finish')
-            _('Commit')
+        _('&Next >')
+        _('< &Back')
+        _('Cancel')
+        _('&Finish')
+        _('Commit')
 
     def __init__(self, parent):
         QWizard.__init__(self, parent)
-        self.setWindowTitle(__appname__+' '+_('welcome wizard'))
-        p  = QPixmap()
-        p.loadFromData(open(P('content_server/calibre.png'), 'rb').read())
-        self.setPixmap(self.LogoPixmap, p.scaledToHeight(80,
-            Qt.SmoothTransformation))
-        self.setPixmap(self.WatermarkPixmap,
-            QPixmap(I('welcome_wizard.png')))
-        self.setPixmap(self.BackgroundPixmap, QPixmap(I('wizard.png')))
+        self.setWindowTitle(__appname__+' '+_('Welcome wizard'))
+        self.setPixmap(self.LogoPixmap, QIcon(I('library.png')).pixmap(48, 48))
+        self.setWizardStyle(self.ModernStyle)
         self.device_page = DevicePage()
         self.library_page = LibraryPage()
         self.library_page.retranslate.connect(self.retranslate)
@@ -807,18 +900,11 @@ class Wizard(QWizard):
         self.setPage(self.stanza_page.ID, self.stanza_page)
 
         self.device_extra_page = None
-        nh, nw = min_available_height()-75, available_width()-30
-        if nh < 0:
-            nh = 580
-        if nw < 0:
-            nw = 400
-        nh = min(400, nh)
-        nw = min(580, nw)
-        self.resize(nw, nh)
         self.set_button_texts()
+        self.resize(600, 520)
 
     def set_button_texts(self):
-        for but, text in self.BUTTON_TEXTS.iteritems():
+        for but, text in iteritems(self.BUTTON_TEXTS):
             self.setButtonText(getattr(self, but+'Button'), _(text))
 
     def retranslate(self):
@@ -835,8 +921,8 @@ class Wizard(QWizard):
         QWizard.accept(self)
 
     def set_finish_text(self, *args):
-        bt = unicode("<em>" + self.buttonText(self.FinishButton) + "</em>").replace('&', '')
-        t = unicode(self.finish_page.finish_text.text())
+        bt = unicode_type("<em>" + self.buttonText(self.FinishButton) + "</em>").replace('&', '')
+        t = unicode_type(self.finish_page.finish_text.text())
         if '%s' in t:
             self.finish_page.finish_text.setText(t%bt)
 
@@ -845,8 +931,8 @@ def wizard(parent=None):
     w = Wizard(parent)
     return w
 
+
 if __name__ == '__main__':
     from calibre.gui2 import Application
     app = Application([])
     wizard().exec_()
-
